#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['chitra',
 'chitra.converter',
 'chitra.serve',
 'chitra.serve.schema',
 'chitra.serve.tf_serving',
 'chitra.serve.torch_serve',
 'chitra.utility',
 'chitra.visualization']

package_data = \
{'': ['*']}

install_requires = \
['tensorflow >= 2.3',
 'tensorflow-addons>=0.13.0',
 'tf-keras-vis>=0.5.3',
 'matplotlib',
 'pillow',
 'imgaug >=0.4.0',
 'requests >=2.24.0,<3.0.0',
 'scikit-learn']

extras_require = \
{'all': ['onnx',
         'onnx2pytorch',
         'tf2onnx',
         'fastapi',
         'uvicorn',
         'pydantic',
         'tensorflow-serving-api',
         'torch']}

setup(name='chitra',
      version='0.1.0b3',
      description='A Deep Learning Computer Vision Utility library',
      author='Aniket Maurya',
      author_email='hello@aniketmaury.com',
      url='https://github.com/aniketmaurya/chitra',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.7',
     )
